package com.sillycycle.bagleyd.tetris;

/*
 * @(#)FieldView.java
 *
 * Copyright 1993 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995 Q. Alex Zhao, azhao AT cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import com.sillycycle.bagleyd.util.Colour;

//import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Point;

/**
 * The <code>Thing</code> class keeps track of the falling shapes.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/altris.html
 */

public class FieldView {
	int             units, polyNumber;
	Point           position = new Point(0, 0);
	int             size, colorNumber, direction;
	int             randomRotation, randomReflection;
	int             randomNumber;
	boolean         bonus;

	//private GameView gameView;
	//private Bitmap[] bmps;
	private int x, y;
	private int width, height;
	int boxSize;
	int boxWidth = boxSize, boxHeight = boxSize;
	int columns, rows;
	Point fieldOffset = new Point(2, 2);
	int background;
	//private Thing curThing;
	private PolyominoImage polyominoImage;
	//private PolyMode[] tris; //[TetrisInterface.MAX_UNITS - MIN_UNITS + 1];
	private Field[][] field = null;

	/*public FieldView(GameView gameView, Bitmap[] bmps) {
		this.gameView = gameView;
		this.bmps = bmps;
	}*/

    private void update() {
	// move thing
        /*if (x >= gameView.getWidth() - width - xSpeed || x + xSpeed <= 0) {
               xSpeed = -xSpeed;
        }
        x = x + xSpeed;
        if (y >= gameView.getHeight() - height - ySpeed || y + ySpeed <= 0) {
               ySpeed = -ySpeed;
        }
        y = y + ySpeed;
        currentFrame = ++currentFrame % BMP_COLUMNS;*/
    }

    public void onDraw(Canvas canvas) {
        update();
        /*int srcX = currentFrame * width;
        int srcY = getAnimationRow() * height;
        Rect src = new Rect(srcX, srcY, srcX + width, srcY + height);
        Rect dst = new Rect(x, y, x + width, y + height);
        canvas.drawBitmap(bmp, src, dst, null);*/
    }

    /*private int getAnimationRow() {
        double dirDouble = (Math.atan2(xSpeed, ySpeed) / (Math.PI / 2) + 2);
        int direction = (int) Math.round(dirDouble) % BMP_ROWS;
        return DIRECTION_TO_ANIMATION_MAP[direction];
    }*/

    public boolean isCollision(float x2, float y2) {
        return x2 > x && x2 < x + width
		&& y2 > y && y2 < y + height;
    }

	void drawField(Canvas canvas, int start, int finish) {
		int i, j;

		for (i = 0; i < columns; i++)
			for (j = start; j < finish; j++) {
				if (field[j][i].pmid >= 0)
					polyominoImage.drawUnit(canvas, false,
						field[j][i].pmid,
						field[j][i].cid,
						fieldOffset.x, fieldOffset.y,
						i, j, boxWidth, boxHeight);
				else
					clearUnit(canvas, i, j);
			}
	}

	void drawField(Canvas canvas) {
		drawField(canvas, 0, rows);
	}


	void clearUnit(Canvas canvas, int i, int j) {
		PolyominoImage.clearUnits(canvas, new Colour(background),
			fieldOffset.x, fieldOffset.y,
			i, j, boxWidth, boxHeight);
	}

	void xorUnit(Canvas canvas, int i, int j, int k) {
		if (field[j][i].pmid >= 0) {
			Colour color = PolyominoImage.colorThings[field[j][i].cid];
			if (k % 2 == 0) // Simulate Xor
				color = new Colour(255 - color.getRed(),
					255 - color.getGreen(),
					255 - color.getBlue());
			polyominoImage.drawUnit(canvas, false, field[j][i].pmid,
				color, fieldOffset.x, fieldOffset.y,
				i, j, boxWidth, boxHeight);
		}
	}

	/*void clearField(Canvas canvas) {
		// This will flash
		PolyominoImage.clearUnits(canvas, new Colour(background), 0, 0, 0, 0,
			getWidth(), getHeight());
		resizeField();
		resetImage();
		for (int i = 0; i < columns; i++)
			for (int j = 0; j < rows; j++)
				clearUnit(canvas, i, j);
		drawFrame(canvas, true);
	}

	void initFill() {
		int i, j, col, fixedUnits;
		int turnStyle = tris[current.units - TetrisInterface.MIN_UNITS].mode
			[(current.diagonal) ? 1 : 0].turnStyle;
		int numberPolyominoes = tris[next.units - TetrisInterface.MIN_UNITS].mode
			[(next.diagonal) ? 1 : 0].number[(next.mixed) ? 1 : 0][turnStyle];

		if (fixedFill) {
			// This algorithm randomly picks fixedUnits per row
			// (4 for DEFAULTWIDTH, ie. 10/2-1)
			// This is more in keeping with the original
			fixedUnits = columns / 2 - 1;
			for (j = 0; j < rows; j++) {
				for (i = 0; i < columns; i++) {
					field[j][i].pmid = -1;
					field[j][i].cid = 0;
				}
				if (j >= rows - fill)
					for (i = 0; i < fixedUnits; i++) {
						do
							col = generator.nextInt(columns);
						while (field[j][col].pmid != -1);
						field[j][col].pmid = PolyominoImage.NUM_BITMAPS - 1;
						field[j][col].cid = generator.nextInt(numberPolyominoes);
					}
			}
		} else {
			// This algorithm randomly fills boxes to a certain level
			//   The problem with this is that it could potentially fill
			//   up a whole row
			for (i = 0; i < columns; i++)
				for (j = 0; j < rows; j++)
					if ((j >= rows - fill) &&
							(generator.nextInt(2) == 0)) {
						field[j][i].pmid = PolyominoImage.NUM_BITMAPS - 1;
						field[j][i].cid = generator.nextInt(numberPolyominoes);
					} else {
						field[j][i].pmid = -1;
						field[j][i].cid = 0;
					}
		}
	}

	void fillLines(Canvas canvas) {
		int i, j;
		int turnStyle = tris[current.units - TetrisInterface.MIN_UNITS].mode
			[(current.diagonal) ? 1 : 0].turnStyle;

		for (j = 0; j <= rows / 2; j++) {
			for (i = 0; i < columns; i++) {
				polyominoImage.drawUnit(canvas, false,
					PolyominoImage.NUM_BITMAPS - 1,
				  generator.nextInt(tris[current.units - TetrisInterface.MIN_UNITS].mode
				  [(current.diagonal) ? 1 : 0].number[(current.mixed) ? 1 : 0]
				  [turnStyle]), fieldOffset.x, fieldOffset.y,
				  i, j, boxWidth, boxHeight);
				polyominoImage.drawUnit(canvas, false,
					PolyominoImage.NUM_BITMAPS - 1,
				  generator.nextInt(tris[current.units - TetrisInterface.MIN_UNITS].mode
				  [(current.diagonal) ? 1 : 0].number[(current.mixed) ? 1 : 0]
				  [turnStyle]), fieldOffset.x, fieldOffset.y,
				  i, rows - j - 1, boxWidth, boxHeight);
			}
		}
	}*/

	int checkLines(Canvas canvas) {
		int[] lSet = new int[rows];
		int nSet = 0, i, j, row;

		/* Find filled rows */
		for (j = 0; j < rows; j++) {
			lSet[j] = 0;
			for (i = 0; i < columns; i++)
				if (field[j][i].pmid >= 0)
					lSet[j]++;
			if (lSet[j] == columns)
				nSet++;
		}
		if (nSet == 0) {
			return 0;
		}
		for (int k = 0; k < 2 * TetrisInterface.NUM_FLASHES; k++) {
		  for (j = 0; j < rows; j++) {
		    if (lSet[j] == columns)
		      for (i = 0; i < columns; i++) {
			xorUnit(canvas, i, j, k);
		      }
		  }
		  try {
		    Thread.sleep(100L);
		  } catch (Exception e) {
		    e.printStackTrace();
		  }
		}
		/* Shrink */
		for (j = rows - 1; j >= 0; j--)
			if (lSet[j] == columns) {
			  for (row = j; row > 0; row--)
			    for (i = 0; i < columns; i++) {
			      field[row][i].pmid = field[row - 1][i].pmid;
			      field[row][i].cid = field[row - 1][i].cid;
			    }
			  for (i = 0; i < columns; i++)
			    field[0][i].pmid = -1;
			  drawField(canvas, 0, j + 1);

			  for (i = j; i > 0; i--)
			    lSet[i] = lSet[i - 1];
			  lSet[0] = 0;

			  if (j > 0)
			    for (i = 0; i < columns; i++) {
			      int tmp = field[j][i].pmid;
			      if ((tmp >= 0) && (tmp != PolyominoImage.CHECKDOWN(tmp))) {
				field[j][i].pmid = PolyominoImage.CHECKDOWN(tmp);
				polyominoImage.drawUnit(canvas, false,
				  field[j][i].pmid, field[j][i].cid,
				  fieldOffset.x, fieldOffset.y,
				  i, j, boxWidth, boxHeight);
			      }
			    }
			  j++;
			  if (j < rows)
			    for (i = 0; i < columns; i++) {
			      int tmp = field[j][i].pmid;

			      if ((tmp >= 0) && (tmp != PolyominoImage.CHECKUP(tmp))) {
				field[j][i].pmid = PolyominoImage.CHECKUP(tmp);
				polyominoImage.drawUnit(canvas, false,
				  field[j][i].pmid, field[j][i].cid,
				  fieldOffset.x, fieldOffset.y,
				  i, j, boxWidth, boxHeight);
			      }
			    }
			  /*if (((TetrisApplet)applet).getToggleSound())
			    ((TetrisApplet)applet).playBumpAudio();*/
			}
		return nSet;
	}

	class Field {
		int pmid, cid; // pixmaps and colors

		public Field() {
			pmid = -1;
			cid = 0;
		}
	}

}
